/* -*- mode: c++; c-basic-offset: 3; -*- */
#ifndef DQ_BIT_HH
#define DQ_BIT_HH 1

#include "TSeries.hh"
#include <iosfwd>
#include <map>
#include <vector>
#include <string>
#include <cmath>


/**  DQ_bit is a base class for classes that return a boolean state that 
  *  will be recorded as a bit in a specified channel name and bit number.
  */
class DQ_bit {
public:
   typedef std::vector<std::string> chanlist_type;
   typedef std::vector<TSeries>     tser_vect;
public:
   DQ_bit(void);
   virtual ~DQ_bit(void);
   const std::string& bit_channel(void) const {return _flag_channel;}
   void bit_channel(const std::string& chan);
   int bit_number(void) const {return _flag_bit;}
   void bit_number(int bit);
   virtual bool bit_value(const tser_vect& data)=0;
   const chanlist_type& channel_list(void) const {return _chan_list;}
   void channel_list(const chanlist_type& chans);
   const std::string& dq_class(void) const {return _dq_class;}
   void dq_class(const std::string& dqclass);
   const std::string& dq_name(void) const {return _name;}
   void dq_name(const std::string& dqname);
   virtual void init(void);
   double numeric_param(const std::string& name) const;
   void numeric_param(const std::string& name, double val);
   const std::string& plugin(void) const;
   void put_config(std::ostream& out) const;
   const std::string& string_param(const std::string& name) const;
   void string_param(const std::string& name, const std::string& val);

private:
   std::string   _name;
   std::string   _dq_class;
   chanlist_type _chan_list;
   int           _flag_bit;
   std::string   _flag_channel;
protected:
   typedef std::map<std::string, double>      num_par_type;
   typedef num_par_type::iterator num_par_iter;
   typedef num_par_type::const_iterator const_num_par_iter;
   num_par_type _num_par;
   typedef std::map<std::string, std::string> str_par_type;
   typedef str_par_type::iterator str_par_iter;
   typedef str_par_type::const_iterator const_str_par_iter;
   str_par_type _str_par;
};

/** Define a plugin definition macro.
 */
#define DQ_PLUGIN(name) extern "C" DQ_bit* name##_init(void) {return new name;}

#endif
